import logging
from typing import Any, List
from urllib.parse import urljoin

from aioze.access import Access

_LOGGER = logging.getLogger(__package__)


class UserInfo:
    def __init__(self, access: Access):
        self._access = access

    async def get_user_info(self):
        """Get authenticated user information"""
        if not await self._access.is_authenticated():
            await self._access.authenticate()
        user_info_url = urljoin(self._access.api_url, "v1/users/me")
        _LOGGER.debug("Getting '%s'", user_info_url)
        res = await self._access.api_wrapper("get", user_info_url)
        if res.status != 200:
            _LOGGER.error("Error fetching Oze user information: '%s'", res.reason)
            return
        return await res.json()

    @staticmethod
    def get_pupils_from_userinfo(user_info: dict[str, Any]) -> List[dict[str, str]]:
        pupils = []
        for pupil in user_info['relations']:
            pupils.append({
                "uid": pupil['user']['id'],
                "last_name": pupil['user']['nom'],
                "first_name": pupil['user']['prenom'],
                "etab": user_info['currentUai']
            })
        return pupils



# -*- coding:utf-8 -*-

__all__ = []

from aioze.api.event import Event
from aioze.api.punishment import Punishment
from aioze.api.notification import Notification
from aioze.api.information import Information
from aioze.api.homework import Homework
from aioze.api.userinfo import UserInfo
from aioze.api.email import Email



import asyncio
import socket
import aiohttp
import async_timeout
import logging
from urllib.parse import urljoin

_LOGGER = logging.getLogger(__package__)


class Access:
    """Connection control."""

    def __init__(self, session: aiohttp.ClientSession, base_url, api_url, username, password, timeout):
        """Init class."""
        self.session = session
        self.base_url = base_url
        self.api_url = api_url
        self.username = username
        self.password = password
        self.timeout = timeout
        self.profil = None
        self.etab = None

    async def is_authenticated(self) -> bool:
        """Check if session is authenticated to the OzE instance"""
        res = await self.api_wrapper("get", urljoin(self.api_url, "v1/users/me"))
        _LOGGER.debug("res: %s", res)
        _LOGGER.debug("req: %s", res.request_info)
        if res.status != 200:
            return False
        info: dict = await res.json()
        self.profil = info['currentProfil']['codeProfil']
        self.etab = info['currentUai']
        _LOGGER.debug("JSON result: '%s'", info)
        return True

    async def authenticate(self) -> bool:
        """Authenticate to the OzE instance"""
        if await self.is_authenticated():
            return True
        # Get the necessary cookies (in my testing, both GETs seemed to be needed)
        res = await self.api_wrapper("get", self.base_url)
        _LOGGER.debug("res: %s", res)
        _LOGGER.debug("req: %s", res.request_info)
        res = await self.api_wrapper("get", urljoin(self.base_url, "my.policy"))
        _LOGGER.debug("res: %s", res)
        _LOGGER.debug("req: %s", res.request_info)
        # POST login information
        res = await self.api_wrapper(
            "post",
            urljoin(self.base_url, "my.policy"),
            data={
                "username": self.username,
                "password": self.password,
                "fakepassword": "fake",
                "private": "prive",
                "vhost": "standard",
                "SubmitCreds.x": "196",
                "SubmitCreds.y": "26",
            },
        )
        _LOGGER.debug("res: %s", res)
        _LOGGER.debug("req: %s", res.request_info)
        return await self.is_authenticated()

    async def api_wrapper(
        self, method: str, url: str, data: dict = {}, headers: dict = {}, params: dict = {}
    ) -> aiohttp.ClientResponse:
        """Get information from the API."""
        try:
            async with async_timeout.timeout(self.timeout):
                if method == "get":
                    return await self.session.get(url, headers=headers, params=params, allow_redirects=False)

                elif method == "post":
                    return await self.session.post(url, headers=headers, data=data, allow_redirects=False)

        except asyncio.TimeoutError as exception:
            _LOGGER.error(
                "Timeout error fetching information from %s - %s",
                url,
                exception,
            )
            raise

        except (KeyError, TypeError) as exception:
            _LOGGER.error(
                "Error parsing information from %s - %s",
                url,
                exception,
            )
            raise

        except (aiohttp.ClientError, socket.gaierror) as exception:
            _LOGGER.error(
                "Error fetching information from %s - %s",
                url,
                exception,
            )
            raise

        except Exception as exception:  # pylint: disable=broad-except
            _LOGGER.error("Something really wrong happened! - %s", exception)
            raise



import asyncio
import inspect
import socket
from urllib.parse import urlparse, urlunparse
import aiohttp
from aioze.access import Access
import aioze.api as Api

TIMEOUT = 10

HEADERS = {"Content-type": "application/json; charset=UTF-8"}


class OzeApiClient:
    """Deals with Oze API calls"""

    def __init__(
        self, url: str, username: str, password: str, session: aiohttp.ClientSession
    ):
        """Sample API Client."""
        self._access = None
        self._url = url
        self._username = username
        self._password = password
        self._session = session

    def _load_modules(self):
        """Instantiate modules."""
        for name in Api.__dict__.keys():
            obj = getattr(Api, name)
            if inspect.isclass(obj):
                setattr(self, name.lower(), obj(self._access))

    async def connect(self):
        api_url = urlunparse(urlparse(self._url)._replace(netloc="api-" + urlparse(self._url).netloc))
        self._access = Access(
            session=self._session,
            base_url=self._url,
            api_url=api_url,
            username=self._username,
            password=self._password,
            timeout=TIMEOUT,
        )

        if self._access:
            # pylint: disable=unused-variable
            try:
                await self._access.authenticate()
            except asyncio.TimeoutError as exception:
                raise
            except (KeyError, TypeError) as exception:
                raise
            except (aiohttp.ClientError, socket.gaierror) as exception:
                raise
            except Exception as exception:  # pylint: disable=broad-except
                raise

            self._load_modules()

        return await self._access.is_authenticated()
